#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# This file is part of wps-dictionary-installer
#
# Copyright (C) 2016
# Lorenzo Carbonell Cerezo <lorenzo.carbonell.cerezo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import locale
import gettext
import sys

__author__ = 'Lorenzo Carbonell <lorenzo.carbonell.cerezo@gmail.com>'
__date__ = '$07/01/2012'
__copyright__ = 'Copyright (c) 2011 Lorenzo Carbonell'
__license__ = 'GPLV3'
__url__ = 'http://www.atareao.es'
__version__ = '0.0.2.0'

######################################


def is_package():
    return __file__.find('src') < 0

######################################


VERSION = __version__
APP = 'wps-dictionary-installer'
APPNAME = 'wps-dictionary-installer'
APP_CONF = APP + '.conf'
CONFIG_DIR = os.path.join(os.path.expanduser('~'), '.config')
CONFIG_APP_DIR = os.path.join(CONFIG_DIR, APP)
# check if running from source
if is_package():
    ROOTDIR = '/opt/extras.ubuntu.com/wps-dictionary-installer'
    APPDIR = os.path.join(ROOTDIR, 'share/wps-dictionary-installer')
    DICTSDIR = os.path.join(APPDIR, 'dicts')
    LANGDIR = os.path.join(ROOTDIR, 'locale-langpack')
    BINDIR = os.path.join(ROOTDIR, 'bin')
    ICONDIR = os.path.join(ROOTDIR, 'share/icons')
    PIXMAPDIR = os.path.join(ROOTDIR, 'share/pixmaps')
else:
    VERSION = VERSION + '-src'
    ROOTDIR = os.path.dirname(__file__)
    LANGDIR = os.path.normpath(os.path.join(ROOTDIR, '../template1'))
    APPDIR = ROOTDIR
    ICONDIR = os.path.normpath(os.path.join(ROOTDIR, '../data'))
    DICTSDIR = os.path.normpath(os.path.join(ROOTDIR, '../data/dicts'))
    PIXMAPDIR = ICONDIR
#
CODESFILE = os.path.join(APPDIR, 'codes.json')
ICON = os.path.join(ICONDIR, 'wps-dictionary-installer.png')

try:
    current_locale, encoding = locale.getdefaultlocale()
    language = gettext.translation(APP, LANGDIR, [current_locale])
    language.install()
    if sys.version_info[0] == 3:
        _ = language.gettext
    else:
        _ = language.ugettext
except Exception as e:
    print(e)
    _ = str
