#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# This file is part of wps-dictionary-installer
#
# Copyright (C) 2016
# Lorenzo Carbonell Cerezo <lorenzo.carbonell.cerezo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gi
try:
    gi.require_version('Gtk', '3.0')
except Exception as e:
    print(e)
    exit(-1)
from gi.repository import Gtk
import requests
import urllib.request
from urllib.parse import urljoin
from urllib.parse import unquote_plus
import lxml
import lxml.html
import os
import glob
import tempfile
import shutil
import zipfile
import json
import codecs
import comun

WPS_URL = 'http://wps-community.org/download/dicts/'
LO_URL = 'https://github.com/sbosio/rla-es/releases/download'
FOLDER = '/opt/kingsoft/wps-office/office6/dicts/spellcheck/'
LO_VERSION = 'v2.3'
LO_DICT = ['es-AR', 'es-BO', 'es-CL', 'es-CO', 'es-CR', 'es-CU', 'es-DO',
           'es-EC', 'es-ES', 'es-GT', 'es-HN', 'es-MX', 'es-NI', 'es-PA',
           'es-PE', 'es-PR', 'es-PY', 'es-SV', 'es-UY', 'es-VE']


def get_codes():
    try:
        f = codecs.open(comun.CODESFILE, 'r', 'utf-8')
        codes = json.loads(f.read())
    except Exception as e:
        print(e)
        codes = {}
    finally:
        f.close()
    return codes


def get_dictionaries():
    onedicts = get_file_urls(WPS_URL, 'zip')
    for adict in onedicts:
        ccode = 'es-{0}'.format(adict['code'][3:].upper())
        if ccode in LO_DICT:
            onedicts.remove(adict)
    gotdicts = []
    for adict in onedicts:
        gotdicts.append(adict['code'].lower())
    print('========== gotdicts ============')
    print(gotdicts)
    print('========== onedicts ============')
    print(onedicts)
    lo = []
    codes = get_codes()['codes']
    codes_code = [k['code'] for k in codes]
    for local_dict in LO_DICT:
        if local_dict.lower() in codes_code:
            selected = [d for d in codes if d['code'] == local_dict.lower()][0]
            new_dict = {}
            new_dict['code'] = selected['code']
            filename = 'es_{0}.oxt'.format(selected['code'][3:].upper())
            new_dict['url'] = '{0}/{1}/{2}'.format(LO_URL,
                                                   LO_VERSION,
                                                   filename)
            new_dict['name'] = selected['name']
            lo.append(new_dict)
    print(lo)
    for index, adict in enumerate(lo):
        if adict['code'].lower() not in gotdicts:
            gotdicts.append(adict['code'].lower())
            onedicts.append(adict)
    sorted_values = sorted(onedicts, key=lambda k: k['name'])
    print(sorted_values)
    return sorted_values


def name_file(imgUrl):
    fileName = unquote_plus(unquote_plus(unquote_plus(
        os.path.basename(imgUrl))))
    if os.path.exists(fileName):
        base, ext = os.path.splitext(fileName)
        print(fileName)
        print(base + '*' + ext)
        nfiles = len(glob.glob(base + '*' + ext))
        fileName = base + '_' + str(nfiles) + ext
    return fileName


def get_file_urls(mainUrl, extension):
    uniFileUrls = []
    if not mainUrl.lower().startswith('http://') and not\
            mainUrl.lower().startswith('https://'):
        mainUrl = 'http://%s' % mainUrl
    print('Downloading from %s...' % mainUrl)
    if extension.startswith('*'):
        extension = extension[1:]
    if not extension.startswith('.'):
        extension = '.' + extension
    req = urllib.request.Request(
        mainUrl,
        data=None,
        headers={
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) \
AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36'
        }
    )
    urlContent = urllib.request.urlopen(req).read().decode('utf-8')
    html = lxml.html.fromstring(urlContent)
    urls = html.xpath('//a/@href')
    for url in urls:
        if url.endswith(extension):
            url = urljoin(mainUrl, url)
            if url not in uniFileUrls:
                newurl = {}
                fileName = name_file(url)
                basename, extension = os.path.splitext(fileName)
                basename = basename.lower().replace('_', '-')
                codes = get_codes()['codes']
                codes_code = [k['code'] for k in codes]
                if basename in codes_code:
                    selected = [d for d in codes if d['code'] == basename][0]
                    newurl['code'] = selected['code']
                    newurl['url'] = url
                    newurl['name'] = selected['name']
                    uniFileUrls.append(newurl)
    return uniFileUrls


def get_selected_value_in_combo(combo):
    model = combo.get_model()
    return (model.get_value(combo.get_active_iter(), 0),
            model.get_value(combo.get_active_iter(), 1),
            model.get_value(combo.get_active_iter(), 2))


def download_file(filename, url):
    headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS \
X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 \
Safari/537.36'}
    try:
        filename = os.path.join(filename, name_file(url))
        r = requests.get(url, stream=True, headers=headers)
        print('----', url, filename, '----')
        with open(filename, 'wb') as f:
            for chunk in r.iter_content(chunk_size=1024):
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)
        return filename
    except Exception as e:
        print(e)
    return None


class SL(Gtk.Dialog):  # needs GTK, Python, Webkit-GTK
    def __init__(self):
        Gtk.Dialog.__init__(
            self,
            'WPS Dictionary Downloader',
            None,
            Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
             Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
        self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
        self.set_size_request(400, 100)
        self.set_icon_from_file(comun.ICON)
        self.connect('destroy', self.close_application)
        #
        grid = Gtk.Grid()
        grid.set_row_spacing(5)
        grid.set_column_spacing(5)
        grid.set_border_width(5)
        self.get_content_area().add(grid)
        #
        label10 = Gtk.Label('Select dictionary:')
        label10.set_alignment(0, 0.5)
        grid.add(label10)
        #
        store = Gtk.ListStore(str, str, str)
        for dictionary in get_dictionaries():
            store.append([dictionary['name'],
                          dictionary['url'],
                          dictionary['code']])
        self.tree = Gtk.ComboBox.new_with_model_and_entry(store)
        self.tree.set_entry_text_column(0)
        self.tree.set_active(1)
        grid.attach(self.tree, 1, 0, 1, 1)
        #
        self.show_all()

    def close_application(self, widget, event, data=None):
        exit(0)

    def on_button_clicked(self, widget):
        dialog = Gtk.FileChooserDialog(
            'Select folder',
            None,
            Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
             Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
        dialog.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
        filter = Gtk.FileFilter()
        filter.set_name("Folder")
        filter.add_pattern("*")  # whats the pattern for a folder
        dialog.add_filter(filter)
        if dialog.run() == Gtk.ResponseType.ACCEPT:
            dialog.hide()
            print(dialog.get_filename())
            self.button.set_label(dialog.get_filename())
        dialog.destroy()


def main():
    sl = SL()
    if sl.run() == Gtk.ResponseType.ACCEPT:
        sl.hide()
        ans = get_selected_value_in_combo(sl.tree)
        ans_name = ans[0]
        ans_url = ans[1]
        ans_code = ans[2]
        if not os.path.exists('/tmp'):
            os.mkdir('/tmp')
        dirpath = tempfile.mkdtemp()
        if ans_url == 'local':
            print(ans_code)
            pos = ans_code.find('_')
            if pos > -1:
                code = ans_code[:pos] + '_' + ans_code[pos + 1:].upper()
            elif ans_code.find('-') > -1:
                pos = ans_code.find('-')
                code = ans_code[:pos] + '_' + ans_code[pos + 1:].upper()
            else:
                code = ans_code
            print(code)
            dirpath2 = os.path.join(dirpath, code)
            os.mkdir(dirpath2)
            shutil.move(os.path.join(comun.DICTSDIR, code + '.aff'),
                        os.path.join(dirpath2, 'main.aff'))
            shutil.move(os.path.join(comun.DICTSDIR, code + '.dic'),
                        os.path.join(dirpath2, 'main.dic'))
            fileconf = open(os.path.join(dirpath2, 'dict.conf'), 'w')
            fileconf.write('[Dictionary]\n')
            fileconf.write('DisplayName=%s\n' % (ans_name))
            fileconf.close()
            for afile in os.listdir(dirpath):
                if os.path.isfile(os.path.join(dirpath, afile)):
                    os.remove(os.path.join(dirpath, afile))
                elif afile != code:
                    shutil.rmtree(os.path.join(dirpath, afile))
            dp = os.path.join(FOLDER,
                              ans_code)
            if os.path.exists(dp):
                shutil.rmtree(dp)
            shutil.move(dirpath2, FOLDER)
        else:
            downloaded_file = download_file(dirpath, ans_url)
            if ans_url.endswith('zip'):
                zfile = zipfile.ZipFile(downloaded_file)
                zfile.extractall(dirpath)
                os.remove(downloaded_file)
                basename, extension = os.path.splitext(name_file(
                    downloaded_file))
                print(basename, extension)
                files = ['main.dic', 'main.aff', 'dict.conf']
                for afile in os.listdir(os.path.join(dirpath, basename)):
                    print(afile)
                    if afile not in files:
                        os.remove(os.path.join(dirpath, basename, afile))
                dirpath2 = os.path.join(dirpath, basename)
                dp = os.path.join(FOLDER,
                                  basename)
                if os.path.exists(dp):
                    shutil.rmtree(dp)
                shutil.move(dirpath2,
                            FOLDER)
            elif ans_url.endswith('oxt'):
                zfile = zipfile.ZipFile(downloaded_file)
                zfile.extractall(dirpath)
                os.remove(downloaded_file)
                basename, extension = os.path.splitext(name_file(
                    downloaded_file))
                pos = basename.find('_')
                if pos > -1:
                    code = basename[:pos] + '_' + basename[pos + 1:].upper()
                else:
                    code = basename
                print(code)
                dirpath2 = os.path.join(dirpath, code)
                os.mkdir(dirpath2)
                shutil.move(os.path.join(dirpath, code + '.aff'),
                            os.path.join(dirpath2, 'main.aff'))
                shutil.move(os.path.join(dirpath, code + '.dic'),
                            os.path.join(dirpath2, 'main.dic'))
                fileconf = open(os.path.join(dirpath2, 'dict.conf'), 'w')
                fileconf.write('[Dictionary]\n')
                fileconf.write('DisplayName=%s\n' % (ans_name))
                fileconf.close()
                for afile in os.listdir(dirpath):
                    if os.path.isfile(os.path.join(dirpath, afile)):
                        os.remove(os.path.join(dirpath, afile))
                    elif afile != code:
                        shutil.rmtree(os.path.join(dirpath, afile))
                dp = os.path.join(FOLDER,
                                  basename)
                if os.path.exists(dp):
                    shutil.rmtree(dp)
                shutil.move(dirpath2, FOLDER)
            shutil.rmtree(dirpath)


if __name__ == '__main__':
    main()
    exit(0)
